#include <string>
#include <iostream>
#include "ipaddress.h"
#include "router.h"
#include "packet.h"
 
#include <string.h>

DataLink::DataLink(IPAddress network, int bandwidth=1000, int delay=100)
{
  routers_ = new Router*[DL_MAXHOSTS];
  ipaddresses_ = new IPAddress*[DL_MAXHOSTS];

  
  net_ = network;
  hostCount_ = 0;
  bandwidth_ = bandwidth;
  delay_ = delay;

}

DataLink::~DataLink()
{
  
}

void DataLink::addRouter(Router *rtr, IPAddress a)
{
	
	routers_[hostCount_] = rtr;
	ipaddresses_[hostCount_] = new IPAddress(a);
	hostCount_++;

}

IPAddress DataLink::net()
{
  return net_;
}

void DataLink::listHosts()
{
  cout << " ---------  Routers on " << net_.toStringPrefix() << " ----------" << endl;
  for(unsigned int i = 0; i < hostCount_; i++)
    cout << routers_[i]->name() << " : " << ipaddresses_[i]->toString() << endl;

}

void DataLink::deliverPacket(Packet *p)
{
  p->setDelay(p->delay() + delay_);
  for(unsigned int i =0; i < hostCount_; i++)
    if(*ipaddresses_[i] == p->dst() || p->dst().isBroadcast())
      {
	routers_[i]->deliverPacket(p);
	break;
      }
}
  
void DataLink::deliverPacket(Packet *p, IPAddress nextHop)
{
  // New Behavior: add packet to the delivery Queue "Q"

  Q.push_back(Qentry(p, nextHop));

  /* // Old behavior: deliver immediately 
  p->setDelay(p->delay() + delay_);
  for(unsigned int i =0; i < hostCount_; i++)
    if(*ipaddresses_[i] == nextHop || p->dst().isBroadcast())
      {
	routers_[i]->deliverPacket(p);
	break;
      }
  */

}
  
void DataLink::runTimeSlice(int timeWidth)
{

  /* randomize delivery order */

  for(unsigned int i =0; i < Q.size(); i ++)
    {
      int x = (int) (Q.size()*rand()/(RAND_MAX+1.0));
      int y = (int) (Q.size()*rand()/(RAND_MAX+1.0));
     
      /* Swap the packets in slots x and y */
      Qentry t;
      t = Q[x];
      Q[x] = Q[y];
      Q[y] = t;
      
    }
  
  /* Deliver all the packets in the Q */
  for(unsigned int i = 0; i < Q.size(); i ++)
    {
      Packet *p = Q[i].p;
      IPAddress nxHop = Q[i].nxHop;

      p->setDelay(p->delay() + delay_);
      for(unsigned int i =0; i < hostCount_; i++)
	if(*ipaddresses_[i] == nxHop || p->dst().isBroadcast())
	  {
	    routers_[i]->deliverPacket(p);
	    break;
	  }
    }

  Q.clear();
}

