#ifndef ROUTER__H
#define ROUTER__H

#include <string>
#include <iostream>
#include "ipaddress.h"
#include "packet.h"
#include <queue>
#include <list>

#include <vector>
 
#include <string.h>

class DataLink; // virtual datalinks
class Router;  // virtugal routers
class Qentry; // a structure for outbound packet queues

/* some constants.  these will be eliminated ASAP in favor of 
 flexible sizes or configuration options */
#define DL_MAXHOSTS 5
#define RTR_MAXDATALINKS 5
#define RTR_MAXROUTES 10


/* A class for virtual datalink functionality */
class DataLink {
public:
	DataLink(IPAddress network, int bandwidth=1000, int delay=100);
	~DataLink();

	void addRouter(Router *rtr, IPAddress a);
	void removeRouter(Router *rtr);
	IPAddress net();
	void listHosts();
	void deliverPacket(Packet *p);  // shouldn't be used anymore
	void deliverPacket(Packet *p, IPAddress nextHop);

	void runTimeSlice(int timeWidth); // delivers all packets in Q
	int bandwidth() { return bandwidth_;};
	
private:
	IPAddress net_;
	unsigned int hostCount_;
	Router** routers_;
	IPAddress** ipaddresses_;
	int bandwidth_;
	int delay_;
	
	vector< Qentry> Q;
	

};


/* A class which implements router-like functionality */
class Router {
public: 
	Router(string name);
	~Router();

	void addInterface(DataLink* network, IPAddress address);
	void printInterfaces();
	string name() { return name_; } ;
	
	int destInterface(IPAddress a);
	IPAddress nextHop(IPAddress a);
	void addRoute(IPAddress dst, IPAddress nextHop, int metric);
	void printRoutes();
	int localInterface(IPAddress a);
	int  servicePacket(Packet *p);
	void deliverPacket(Packet *p);
	void runTimeSlice(int time);
	void rippleQueue();
	void handleLocalPacket(Packet *p);

private:

	void queuePacket(Packet *p);
	void delayQueue(list<Qentry>, int timeWidth);

	bool forMe(Packet *p);
	DataLink ** dataLinks_;
	IPAddress ** IPAddresses_;
	unsigned int interfaceCount_;
	string name_;

	vector<Packet*> recvQueue;

	vector< list <Qentry> > sendQ;
	queue<Packet*> localQ;

	IPAddress routeTable_[RTR_MAXROUTES][2];
	int routeCount_;
	vector< int > linkQuotas;
};


/* Used to store packets in outbound queues. */
class Qentry
{

 public:
  Qentry(Packet *pack, IPAddress hop) : p(pack), nxHop(hop) { };
  Qentry() { } ;

  Packet *p;
  IPAddress nxHop;
};

#endif

